@@ -1,44 +1,3 @@
-check_query <- function(prop, type, inconsistent, flow, nodes, evidence, clique_root) {
-  if (prop == "no") {
-    stop("It is not possible to query from a junction tree that ",
-      "hasn't been propagated.", call. = FALSE
-    )
-  }
-  
-  if (type %ni% c("marginal", "joint")) {
-    stop("Type must be 'marginal' or 'joint'.", call. = FALSE)
-  }
-
-  if (type == "joint" && inconsistent) {
-    stop(
-      "It is not possible to make joint queries when ",
-      "there is inconsistent evidence", call. = FALSE
-    )
-  }
-  
-  if (flow == "max") {
-    stop(
-      "It does not make sense to query probablities from a junction",
-      "tree with flow = 'max'. ",
-      "Use 'mpe' to obtain the max configuration.", call. = FALSE
-    )
-  }
-
-  if (any(nodes %in% evidence)) {
-    stop("It is not possible to query probabilities from",
-      "evidence nodes", call. = FALSE)
-  }
-  
-  if (prop == "collect") if (!all(nodes %in% clique_root)) {
-    stop(
-      "All nodes must be in the root clique",
-      "since the junction tree has only collected! ",
-      "See get_clique_root(x) to find the nodes in the root clique.",
-      call. = FALSE
-    )
-  }    
-}
-
 # note: Rows only have a single 1 in collect
 #       this indicates the parent. A column with all
 #       zeroes indicates a leave node.
@@ -59,16 +18,6 @@ parents_jt <- function(x, lvs) {
   return(par)
 }
 
-valid_evidence <- function(dim_names, e) {
-  nemvc <- neq_empt_vector_chr(e)
-  e_conforms_with_dim_names <- !anyNA(mapply(match, e, dim_names[names(e)]))
-  if (e_conforms_with_dim_names && nemvc) {
-    return(TRUE)
-  } else {
-    return(FALSE)
-  }
-}
-
 has_root_node_jt <- function(x) attr(x, "root_node") != ""
 
 new_schedule <- function(cliques_chr, cliques_int, root_node, joint_vars = NULL) {
@@ -109,13 +58,13 @@ new_schedule <- function(cliques_chr, cliques_int, root_node, joint_vars = NULL)
       collect      = collect ,
       distribute   = distribute,
       clique_graph = clique_graph,
-      clique_root  = paste("C", clique_root, sep = "") # TODO: Just return the index
+      clique_root  = paste("C", clique_root, sep = "")
     )
   )
 }
 
 prune_jt <- function(jt) {
-
+  
   direction <- attr(jt, "direction")
   x <- if (direction == "collect") jt$schedule$collect else jt$schedule$distribute
 
@@ -164,33 +113,6 @@ prune_jt <- function(jt) {
   return(jt)
 }
 
-set_evidence_ <- function(x, evidence, inc) {
-  # x: list of (sparse) tables
-  for (k in seq_along(x)) {
-    pot_k <- names(x[[k]])
-    if (inherits(x[[k]], "sparta_unity")) next
-    es_in_ck <- which(names(evidence) %in% pot_k)
-
-    if (neq_empt_int(es_in_ck)) {
-      e        <- evidence[es_in_ck]
-      conform  <- length(pot_k) > length(e)
-      m <- if (conform) {
-        try(sparta::slice(x[[k]], e, drop = TRUE), silent = TRUE)  # possibly a sparta_unity
-      } else {
-        try(sparta::slice(x[[k]], e, drop = FALSE), silent = TRUE)
-      }
-
-      if (inherits(m, "try-error")) {
-        # Now the best guess is a uniform prior
-        m <- sparta::sparta_unity_struct(sparta::dim_names(x[[k]]))
-        inc$inc <- TRUE
-      }
-      x[[k]] <- m      
-    }
-  }
-  return(x)
-}
-
 
 new_jt <- function(x, evidence = NULL, flow = "sum") {
   # x: a charge object returned from compile
@@ -203,7 +125,7 @@ new_jt <- function(x, evidence = NULL, flow = "sum") {
   inc$inc <- attr(x, "inconsistencies")
   if (!is.null(evidence)) charge$C <- set_evidence_(charge$C, evidence, inc)
 
-  schedule  <- new_schedule(cliques, attr(x, "cliques_int"), attr(x, "root_node"), attr(x, "joint_vars"))
+  schedule  <- x$schedule # new_schedule(cliques, attr(x, "cliques_int"), attr(x, "root_node"), attr(x, "joint_vars"))
 
   jt <- list(
     schedule = schedule[1:2], # collect and distribute
@@ -275,10 +197,10 @@ send_messages <- function(jt) {
       # because of of inconsistent evidence wich are transformed to unities.
       # This means, that we just put a uniform prior on the respective potential.
       par_eq_unity <- inherits(jt$charge$C[[C_par_k_name]], "sparta_unity")
-      lvs_eq_unity <- inherits(jt$charge$C[[C_lvs_k_name]], "sparta_unity")
+      lvs_eq_unity <- inherits(jt$charge$C[[C_lvs_k_name]], "sparta_unity")      
       
       if (direction == "collect") {
-
+        
         message_k <- sparta::marg(jt$charge$C[[C_lvs_k_name]], message_k_names, attr(jt, "flow"))
 
         # Update parent potential
@@ -287,32 +209,27 @@ send_messages <- function(jt) {
           # If the parent becomes the null-potential we make it a unity afterall
           # This can happen when there is inconsistent evidence or just if data
           # is very sparse
-          if (ncol(jt$charge$C[[C_par_k_name]]) < 1) {
+          if (!is_scalar(jt$charge$C[[C_par_k_name]]) && ncol(jt$charge$C[[C_par_k_name]]) < 1) {
             attr(jt, "inconsistencies") <- TRUE
             dn <- sparta::dim_names(jt$charge$C[[C_par_k_name]])
             jt$charge$C[[C_par_k_name]] <- sparta::sparta_unity_struct(dn, rank = 1)
           }
-          
         } else if (!lvs_eq_unity && par_eq_unity) {
           # Note here, that the new potential is possibly not defined over all variables in the clique
           # We must take this into account wheen we query variables that are no longer in the potential
           if (is_scalar(message_k)) {
-            # Can be a scalar because of evidence reduction.
+            # Can be a scalar because of evidence reduction. So just update the rank
             jt$charge$C[[C_par_k_name]] <- sparta::mult(jt$charge$C[[C_par_k_name]], message_k)
           } else {
             jt$charge$C[[C_par_k_name]] <- message_k
           }
-          # in the last cases, just leave the parent potential as is since we
-          # only multiply with ones. This will result in "wrong" factors; but it
-          # can only occur if there is inconsistent evidence, and here we have
-          # already taken this into account by saying that prob(evidence) is no longer valid.
         }
 
-        # Update leave potential
+        # Update child potential
         jt$charge$C[[C_lvs_k_name]] <- sparta::div(jt$charge$C[[C_lvs_k_name]], message_k)
         
       }
-
+      
       if (direction == "distribute") {
 
         if (attr(jt, "flow") == "max") {
@@ -349,7 +266,7 @@ send_messages <- function(jt) {
         # Update parent potential
         if (!lvs_eq_unity && !par_eq_unity) {
           jt$charge$C[[C_par_k_name]] <- sparta::mult(jt$charge$C[[C_par_k_name]], message_k)
-          if (ncol(jt$charge$C[[C_par_k_name]]) < 1) {
+          if (!is_scalar(jt$charge$C[[C_par_k_name]]) && ncol(jt$charge$C[[C_par_k_name]]) < 1) {
             attr(jt, "inconsistencies") <- TRUE
             dn <- sparta::dim_names(jt$charge$C[[C_par_k_name]])
             jt$charge$C[[C_par_k_name]] <- sparta::sparta_unity_struct(dn, rank = 1)
@@ -360,7 +277,9 @@ send_messages <- function(jt) {
           } else {
             jt$charge$C[[C_par_k_name]] <- message_k  
           }
-        }         
+        }
+
+        # Update the separator
         jt$charge$S[[paste("S", pk, sep = "")]] <- message_k
         
         if (attr(jt, "flow") == "max") {
@@ -371,5 +290,6 @@ send_messages <- function(jt) {
       }
     }
   }
+
   prune_jt(jt)
 }
